#!/usr/bin/env python
# This program will try to iteratively classify and align a set of 2D particle images without any 3D reconstruction

from EMAN import *
from optparse import OptionParser
from math import *
from os import system
from random import random
import os
import sys
import string

def LOGbegin(ARGV):
	out=open(".emanlog","a")
	b=string.split(ARGV[0],'/')
	ARGV[0]=b[len(b)-1]
	out.write("%d\t%d\t1\t%d\t%s\n"%(os.getpid(),time.time(),os.getppid(),string.join(ARGV," ")))
	out.close()

def LOGend():
	out=open(".emanlog","a")
	out.write("%d\t%d\t2\t-\t\n"%(os.getpid(),time.time()))
	out.close()


def sysprint(a):
	print a
	system(a)

def main():
	progname = os.path.basename(sys.argv[0])
	usage = """Usage: %prog [options] <particle_stack>
	
"""

	parser = OptionParser(usage=usage,version=EMANVERSION)

	parser.add_option("--iter", type="int",help="Number of refinement iterations",default=2)
	parser.add_option("--ncls","-N", type="int",help="Number of final classes to produce",default=40)
	parser.add_option("--nosvd", dest="nosvd", action="store_true", default=False, help="Use straight k-means for classification instead of SVD based vectorization")
	parser.add_option("--ninitcls","-I", type="int",help="Number of initial classes for alignment iterations",default=10)
	parser.add_option("--proc","-P", type="int",help="Processors to use",default=0)
	parser.add_option("--ctfcw","-C", type="string",help="Structure factor file for full CTF correction",default=None)
	
	(options, args) = parser.parse_args()
	if len(args)<1 : parser.error("Input stack required")
	
	LOGbegin(sys.argv)
	
	nptcl=fileCount(args[0])[0]
	print "Analyzing %d particles"%nptcl
	if (nptcl<10) : 
		print("Input file must contain at least 10 particles")
		sys.exit(1)
	
	# initial reference-free classification
	system("rm tmp.snrc.*")
	system("proc2d %s tmp.snrc.hed last=1999"%args[0])
	system("startnrclasses tmp.snrc.hed %d nobad proc=%d"%(max(nptcl/50,5),max(1,options.proc)))
	
	# Sort the averages in order of the number of particles in each
	avgs=readImages("classes.nr.hed",-1,-1)
	avgs=seriesalign2(avgs)

	
	try:
		os.unlink("iter.0.hed")
		os.unlink("iter.0.img")
	except: pass
	for i in avgs: i.writeImage("iter.0.hed",-1)
	
	for i in range(1,options.iter):
		system("rm cls*lst")
		system("proc2d iter.%d.hed cls mraprep"%(i-i))

		# While this does classify, we're really using this to get the 
		# best alignment for each particle
		if options.proc:
			fsp="tmp.%d"%int(random()*999999)
			out=file(fsp,"w")
			for k in range(options.proc): out.write("classesbymra %s iter.%d.hed split refine phase frac=%d/%d\n"%(args[0],i-1,k,options.proc))
			out.close()
			system("runpar file=%s proc=%d"%(fsp,options.proc))
#			os.unlink(fsp)
		else:
			system("classesbymra %s iter.%d.hed split refine phase"%(args[0],i-1))
		
		fsp=os.listdir(".")
		try:
			os.unlink("ptcl.hed")
			os.unlink("ptcl.img")
		except:
			pass
			
		# write the aligned particles to ptcl.n.hed
		for f in fsp:
			if f[:3]=="cls" :
				ni=fileCount(f)[0]
				print f,ni
				a=EMData()
				for im in range(1,ni):
					a.readImage(f,im)
					a.edgeNormalize()
					a.rotateAndTranslate()
					a.writeImage("ptcl.hed",-1)

		# now classify
		if (not options.nosvd) :
			# we make our basis set using SVD on the averages
			system("svdcmp iter.%d.hed basis.%d.hed n=10"%(i-1,i))

			# Now make a 'fp' file containing the vectorization
			# of each aligned particle image
			system("basisfilter ptcl.hed basis.%d.hed footprint=ptcl.fp"%(i))

			# Now classify the particles (k-means) based on their
			# footprints
			system("classes ptcl.fp %d"%(options.ninitcls))

			# now make the class-averages
			try:
				os.unlink("iter.%d.hed"%i)
				os.unlink("iter.%d.img"%i)
			except: pass
			for c in range(options.ninitcls):
				if options.ctfcw!=None : system("ctfavg2 cls%04d.lst iter.%d.hed ctfcw=%s nolog"%(c,i,options.ctfcw))
				else : system("proc2d cls%04d.lst iter.%d.hed average"%(c,i))
		else:
			# non SVD based classification. Doesn't work as well
			system("classifykmeans.py stack=ptcl.hed classes=%d average=iter.%d nosingle"%(options.ninitcls,i))
			
		# now realign and sort the averages
		avgs=readImages("iter.%d.hed"%i,-1,-1)
		avgs=seriesalign2(avgs)
		try:
			os.unlink("iter.%d.hed"%i)
			os.unlink("iter.%d.img"%i)
		except: pass

		for im in avgs: 
#			if i<options.iter-2 : im.cmCenter()
			im.cmCenter()
			im.writeImage("iter.%d.hed"%i,-1)

	if (not options.nosvd) :
		# Now classify the particles (k-means) based on their
		# footprints
		system("classes ptcl.fp %d"%(options.ncls))

		# now make the class-averages
		try:
			os.unlink("iter.final.hed"%i)
			os.unlink("iter.final.img"%i)
		except: pass
		for c in range(options.ncls):
			if options.ctfcw!=None : system("ctfavg2 cls%04d.lst iter.final.hed ctfcw=%s nolog"%(c,options.ctfcw))
			else : system("proc2d cls%04d.lst iter.final.hed average"%(c))
	else:
		system("classifykmeans.py stack=ptcl.hed classes=%d average=iter.final nosingle"%(options.ncls))
		
	LOGend()

def seriesalign2(avgs):
	# ok, this little loop finds the mean center of mass position of all
	# aligned particles, then recenters using this mean
	orig=avgs[:]	# copy since seriesalign is destructive
	avgs2=seriesalign(avgs)
	xc,yc,nc=0,0,0
	for im in avgs2:
		im.cmCenter()
		xc+=im.Dx()
		yc+=im.Dy()
		nc+=1
	avgs2[0].setRAlign(0,0,0)
	avgs2[0].setTAlign(xc/nc,yc/nc,0)
	avgs2[0].rotateAndTranslate()	# the first image in avgs2 is the original, not a copy

	return seriesalign(orig)

def seriesalign(avgs) :
	avgs.sort(lambda x,y: cmp(y.NImg(),x.NImg()))
	
	# now align the averages in order by their mutual similarity
	avgsali=[avgs[0]]
	del avgs[0]
	while len(avgs)>0:
		bst=None
		bsti=-1
		bstcmp=1.0e10
		for i in range(len(avgs)):
#			avgs[i].cmCenter()
#			ali=avgs[i].RFAlign(avgsali[-1],None,-1)
			ali=avgs[i].RTFAlign(avgsali[-1],None,1,avgsali[-1].xSize()/6)
			val=ali.lcmp(avgsali[-1])[0]
			if val<bstcmp :
				bstcmp=val
				bsti=i
				bst=ali
		if bst!=None : 
#			print bsti,bstcmp
			avgsali.append(bst)
			del avgs[bsti]
		else : print "Align failed"
	return avgsali
	
main()
